#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

#include "serial.h"
#include "macros.h"

#define TIMEOUT_TIME 5000
// 0 is to the right of the board (with FTDI connector down)
// send_0 = PB2
// send_1 = PA7
// send_2 = PA3
// send_3 = PA2

// 0 is at the top of the board (farthest from FTDI connector)
// receive_0 = PA0
// receive_1 = PA1
// receive_2 = PA4
// receive_3 = PA5

void scan_pad(volatile unsigned char* sendport, volatile unsigned char* sendddr,
              unsigned char sendpin, unsigned char muxval)
{  
    // Wait for the receive pin to settle
    ADMUX = muxval;
    
    int temp_value = 100;
    while(temp_value > 10) {
        ADCSRA |= 1 << ADSC;
        while (ADCSRA & (1 << ADSC));
        temp_value = ADCL;
        temp_value += ADCH << 8;
    }

    // Make receive pins inputs
    clear(DDRA, PA0);
    clear(DDRA, PA1);
    clear(DDRA, PA4);
    clear(DDRA, PA5);
    
    // Activate the first column
    set(*sendddr, sendpin);
    set(*sendport, sendpin);
    
    // Get an analog read
    _delay_us(1);
    ADCSRA |= 1 << ADSC;
    while (ADCSRA & (1 << ADSC));
    put_char(&serial_port, serial_pin_out, ADCH);
    
    clear(*sendport, sendpin);
    clear(*sendddr, sendpin);
    

    // Make receive pins outputs to quickly bleed off charge.
    set(DDRA, PA0);
    set(DDRA, PA1);
    set(DDRA, PA4);
    set(DDRA, PA5);
}

int main(void)
{
    //
    // main
    //
    
    //
    // set clock divider to /1
    //
    CLKPR = (1 << CLKPCE);
    CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);

    // ADC enabled, flag cleared, prescalar set to 128 (156 kHz)
    ADCSRA = 0b10010111;
    set(ADCSRB, ADLAR);

    // Initialize various special features
    serial_init();
    
   const static char m[] PROGMEM = "\nmultitouch board says 'Hello, world'.\n";
    put_flash_string(&serial_port, serial_pin_out, (PGM_P)m);
    
    unsigned char send_pins[4] = {PB2, PA7, PA3, PA2};
    
    unsigned char receive_muxes[4] = {0b00000, 0b000001, 0b000100, 0b000101};
    
    while(1) {
        int send_index, receive_index;
        for (receive_index = 0; receive_index < 4; receive_index++) {
            for (send_index = 0; send_index < 4; send_index++) {

                    if (send_index == 0)
                        scan_pad(&PORTB, &DDRB, PB2, receive_muxes[receive_index]);
                    else
                        scan_pad(&PORTA, &DDRA, send_pins[send_index],
                                 receive_muxes[receive_index]);
            }
        }
//        _delay_ms(12.1);
        _delay_ms(3.76);
        put_char(&serial_port, serial_pin_out, '\n');
    }
    
}
