import processing.serial.*;

Serial myPort;  // Create object from Serial class
int RUNNING_AVERAGE_SIZE = 2;

int[][][] readings = new int[4][4][RUNNING_AVERAGE_SIZE];
int average_pos = 0;

void setup()
{
  size(1000, 600, P3D);
  String portName = Serial.list()[1];
  println(portName);
  myPort = new Serial(this, portName, 115200); 
  myPort.clear();
  smooth();
}

void draw_pads()
{
  float[][] panel_heights = new float[4][4];
  float min_panel = 1000;
  for (int i = 0; i < 4; ++i) {
    for (int j = 0; j < 4; ++j) {
      for (int k = 0; k < RUNNING_AVERAGE_SIZE; ++k)
        panel_heights[i][j] += readings[3-i][j][k];
      panel_heights[i][j] /= float(RUNNING_AVERAGE_SIZE);
      panel_heights[i][j] = (70 - panel_heights[i][j])  / 40.;
      if (panel_heights[i][j] < min_panel)
        min_panel = panel_heights[i][j];
    }
  }
  
  stroke(255);
  translate(width/2., height/2., 0);
  scale(height/4. * 0.9, height/4. * 0.9, height/4. * 0.9);
  translate(0, 0, -1);
  rotateX(radians(45));
  rotateZ(radians(5));
  
  for (int i = 0; i < 4; ++i) {
    for (int j = 0; j < 4; ++j) {
      float panel_height = panel_heights[i][j] - min_panel;

      pushMatrix();
      translate(i - 1.5, j - 1.5, 0);

      if (panel_height > 0.6) {
        stroke(0, 255, 0);
        strokeWeight(5);
      } else {
        stroke(64 + 128 * panel_height);
        strokeWeight(1);
      }
        
        fill(255, (panel_height - 0.4) / 0.4 * 255.0);

      beginShape(QUADS);
      vertex(-0.5, -0.5, panel_height);
      vertex( 0.5, -0.5, panel_height);
      vertex( 0.5,  0.5, panel_height);
      vertex(-0.5,  0.5, panel_height);
      endShape();
      
      popMatrix();
    }
  }
}

void draw()
{
  background(0);
  noFill();

  char val = 0, temp = 0;
  int pad_reading = 0;
  
  int i = 0, j = 0;

  myPort.clear();

  // Wait we get a sync frame
  do
  {
    while(myPort.available() == 0);
  } while (myPort.readChar() != '\n');

   
  while (j < 4) {
    while(myPort.available() == 0);
    readings[i][j][average_pos] = myPort.read();
    if (++i == 4) {
      i = 0;
      j++;
    }
  }


    draw_pads();
  
    if (++average_pos == RUNNING_AVERAGE_SIZE)
      average_pos = 0;
      
  
}
      
